; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 14 - Obstacle Avoidance Robot
; Link: https://www.circuitbread.com/tutorials/bluetooth-controlled-robot
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
PSA         EQU 0003h
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h

PS0         EQU 0000h
PS1         EQU 0001h
PS2         EQU 0002h

;----- STATUS Bits -----------------------------------------------------
C           EQU 0000h
Z           EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i           EQU 10   ;Delay register 1
j           EQU 11   ;Delay register 2
servo1      EQU 12   ;Servo1 pulse width
servo2      EQU 13   ;Servo2 pulse width
steps_back  EQU 14   ;Number of steps to move backward


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~((1<<T0CS)|(1<<PSA)|(1<<PS0))
    OPTION              ;Enable GP2
    MOVLW ~((1 << GPIO_GP0_POSITION) | (1 << GPIO_GP1_POSITION) | (1 << GPIO_GP2_POSITION))
    TRIS GPIO           ;Set GP0, GP1, and GP2 as outputs
LOOP:
    BSF GPIO, GPIO_GP1_POSITION     ;Set GP1 high
    GOTO $+1            ;Perform 10 us delay
    GOTO $+1            ;by making small 2us delays
    GOTO $+1            ;of each GOTO instruction
    GOTO $+1
    GOTO $+1
    BCF GPIO, GPIO_GP1_POSITION     ;Set GP1 low
    MOVLW 3             ;Load 3 into W: (3-1)x128us=256, 256/59=4.3cm
    BTFSS GPIO, GPIO_GP3_POSITION   ;Check until GP3 (Echo) goes high
    GOTO $-1
    CLRF TMR0           ;Clear Timer0
    BTFSC GPIO, GPIO_GP3_POSITION   ;Check until GP3 goes low
    GOTO $-1
    SUBWF TMR0, W       ;Subtract W from TMR0 to check if the
    BTFSS STATUS, C     ;TMR0 is bigger than 2
    GOTO MOVE_BACKWARD  ;If yes the go to the "MOVE_BACKWARD" label
MOVE_FORWARD:           ;Move the robot forward
    MOVLW 220           ;Load the delay value for the servo 1
    MOVWF servo1
    MOVLW 200           ;Load the delay value for the servo 2
    MOVWF servo2
    CALL CONTROL_SERVO  ;Call the CONTROL_SERVO subroutine
    GOTO LOOP           ;and return to the main loop beginning
MOVE_BACKWARD:          ;Move robot backward
    MOVLW 50            ;Load the number of steps to move backward
    MOVWF steps_back    ;And copy it into the 'steps_back' register
LOOP_BACKWARD:          ;Implement the number of 'steps_back' steps
    MOVLW 206           ;Load the delay value for the servo 1
    MOVWF servo1
    MOVLW 220           ;Load the delay value for the servo 2
    MOVWF servo2
    CALL CONTROL_SERVO  ;Call the CONTROL_SERVO subroutine
    DECFSZ steps_back, F;Decrease the 'steps_back' and check if it's 0
    GOTO LOOP_BACKWARD  ;If not then go to the LOOP_BACKWARD label
    GOTO LOOP           ;Otherwise return to the main loop beginning

CONTROL_SERVO:          ;Control the servo 1
    BSF GPIO, GPIO_GP2_POSITION     ;Set GP2 high
    MOVLW 2             ;Load 2 into the second delay register 'j'
    MOVWF j
    MOVF servo1, W      ;Copy the value of the servo1 into the W
    CALL DELAY          ;and call the delay
    BCF GPIO, GPIO_GP2_POSITION   ;Then set GP2 low
    NOP                 ;One cycle delay before the BSF instruction
SERVO_2:                ;Control the servo 2
    BSF GPIO, GPIO_GP0_POSITION   ;Set GP0 high
    MOVLW 2             ;Load 2 into the second delay register 'j'
    MOVWF j
    MOVF servo2, W      ;Copy the value of the servo2 into the W
    CALL DELAY          ;and call the delay
    BCF GPIO, GPIO_GP0_POSITION  ;Then set GP0 low
PAUSE:                  ;Pause between the pulses
    MOVLW 25            ;Load 25 into the second delay register 'j'
    MOVWF j
    CALL DELAY          ;and call the delay
    RETLW 0

DELAY:                  ;Start DELAY subroutine here
    MOVWF i             ;Load the W value into the 'i' register
DELAY_LOOP:
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    DECFSZ j, F         ;Decrement j and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    RETLW 0             ;Else return from the subroutine

    END INIT        ; Program entry point.